/********************************************************************
* 		copyright (C) 2014 all rights reserved
*			 @file: mtd.c
* 		  @Created: 2014-8-8 17:00
* 	  	   @Author: conway chen
* 	  @Description: erase, read and write flash 
*	  @Modify Date: 2014-8-8 17:00
*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <mtd/mtd-user.h>

/**
 * @brief: main function  
 * @Param: argc: number of parameters
 * @Param: argv: parameters list
 */
int main(int argc, char *argv[])
{
    mtd_info_t mtd_info;           
    erase_info_t ei;               
    int i;
    int fd;

    unsigned char data[16] = { 0X00, 0x01, 0x02, 0x03, 
    						   0x04, 0x05, 0x06, 0x07, 
    						   0x08, 0x09, 0x0A, 0x0B, 
    						   0x0C, 0x0D, 0x0E, 0x0F };
    unsigned char read_buf[16] = {0x00};                
    
	if (argc < 2) {
		printf("Usage: %s [MTD device]\n", argv[0]);
		exit(0);	
	}
	/* open the mtd device for reading and writing */
    fd = open(argv[1], O_RDWR); 
   	if (fd < 0) {
   		perror("open");
   		exit(0);	
   	}
   	
	/* get the device info */ 
    ioctl(fd, MEMGETINFO, &mtd_info);       
    printf("MTD Type: %d\nMTD total size: %ld bytes\nMTD erase size: %ld bytes\nMTD write size: %ld bytes\n",
         mtd_info.type, mtd_info.size, mtd_info.erasesize, mtd_info.writesize);
	/* set the erase block size */
    ei.length = mtd_info.erasesize;    
    printf("Erasing the whole flash. Please wait!\n");
    
    for(ei.start = 0; ei.start < mtd_info.size; ei.start += ei.length)
    {
        ioctl(fd, MEMUNLOCK, &ei);        
        printf(".");
        fflush(stdout);
        ioctl(fd, MEMERASE, &ei);
    }
    
    printf("\n");
    printf("erase done!\n");
    /* go back to first block's start */
    lseek(fd, 0, SEEK_SET);               
    printf("writing 16 bytes data to flash...\n");
    /* write data */
    write(fd, data, sizeof(data));
      
    for(i = 0; i<16; i++)
         printf("0x%02x ", data[i]);
         
	printf("write done!\n");
	/* go back to first block's start */
    lseek(fd, 0, SEEK_SET);              
    printf("reading data...\n");
    /* read the data */
    read(fd, read_buf, sizeof(read_buf)); 
            
    for(i = 0; i<16; i++)
         printf("0x%02x ", (unsigned int)read_buf[i]);
         
    printf("\n");
    printf("read done!\n");

    close(fd);
    return 0;
}














 

