/********************************************************************
* 		copyright (C) 2014 all rights reserved
*			 @file: mtd.c
* 		  @Created: 2014-8-8 17:00
* 	  	   @Author: conway chen
* 	  @Description: erase, read and write flash 
*	  @Modify Date: 2014-8-8 17:00
*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <mtd/mtd-user.h>

#define DATA_LENGTH 16

/**
 * @brief: main function  
 * @Param: argc: number of parameters
 * @Param: argv: parameters list
 */
int main(int argc, char *argv[])
{
    mtd_info_t mtd_info;           
    erase_info_t ei; 
    char *write_buf;              
    int i;
    int fd;
    
    unsigned char data[DATA_LENGTH] = { 0x00, 0x01, 0x02, 0x03, 
    						   		 	0x04, 0x05, 0x06, 0x07, 
    						   		 	0x08, 0x09, 0x0A, 0x0B, 
    						   		 	0x0C, 0x0D, 0x0E, 0x0F };
    unsigned char read_buf[DATA_LENGTH] = {0x00};                
    
	if (argc < 2) {
		printf("Usage: %s [MTD device]\n", argv[0]);
		exit(0);	
	}
	/* open the mtd device for reading and writing */
    fd = open(argv[1], O_RDWR); 
   	if (fd < 0) {
   		perror("open");
   		exit(0);	
   	}
   	
	/* get the device info */ 
    ioctl(fd, MEMGETINFO, &mtd_info);       
    printf("\nMTD Type: %d\nMTD total size: %ld bytes\nMTD erase size: %ld bytes\nMTD write size: %ld bytes\n\n",
         mtd_info.type, mtd_info.size, mtd_info.erasesize, mtd_info.writesize);
	/* set the erase block size */
    ei.length = mtd_info.erasesize;
    /* erase the last block */
    ei.start = mtd_info.size - mtd_info.erasesize;    
    ioctl(fd, MEMUNLOCK, &ei);        
    printf("erase the last block at %ld\n", ei.start);
    fflush(stdout);
    ioctl(fd, MEMERASE, &ei);
        
    
    printf("\nerase done!\n\n");
    
    write_buf = malloc(mtd_info.writesize);
    
    for (i = 0; i < DATA_LENGTH; i++) {
    	write_buf[i] = data[i];
    }
    
    /* go back to first block's start */
    lseek(fd, ei.start, SEEK_SET);               
    printf("writing %d bytes data to flash...\n", DATA_LENGTH);
    /* write data to flash */    
    write(fd, write_buf, mtd_info.writesize);    	
    
    for(i = 0; i < DATA_LENGTH; i++)
         printf("0x%02x ", write_buf[i]);
         
	printf("\nwrite done!\n\n");
	/* go back to first block's start */
    lseek(fd, ei.start, SEEK_SET);              
    printf("reading data from flash...\n");
    /* read the data from flash */
    read(fd, read_buf, sizeof(read_buf)); 
            
    for(i = 0; i < DATA_LENGTH; i++)
         printf("0x%02x ", (unsigned int)read_buf[i]);         
    
    printf("\nread done!\n\n");    

    close(fd);
    return 0;
}














 

